module robot_fsm (
        input clk,
        input rst,
        input switch_left,
        input switch_right,
        output reg [7:0] left,
        output reg [7:0] right
    );

    localparam STATE_SIZE = 3;

    localparam FORWARD = 0,
               BACKUP_RIGHT = 1,
               TURN_RIGHT = 2,
               BACKUP_LEFT = 3,
               TURN_LEFT = 4;

    reg [STATE_SIZE-1:0] state_d, state_q;
    reg [25:0] ctr_d, ctr_q;

    reg [1:0] sl_d, sl_q, sr_d, sr_q;

    always @* begin
        sl_d[0] = switch_left;
        sl_d[1] = sl_q[0];
        sr_d[0] = switch_right;
        sr_d[1] = sr_q[0];

        ctr_d = ctr_q + 1'b1;
        state_d = state_q;

        case (state_q)
            FORWARD: begin
                ctr_d = 25'd0;
                if (sr_q[1]) begin
                    state_d = BACKUP_LEFT;
                end else if (sl_q[1]) begin
                    state_d = BACKUP_RIGHT;
                end
            end
            BACKUP_RIGHT: begin
                if (ctr_q == {26{1'b1}})
                    state_d = TURN_RIGHT;
            end
            TURN_RIGHT: begin
                if (ctr_q == {25{1'b1}})
                    state_d = FORWARD;
            end
            BACKUP_LEFT: begin
                if (ctr_q == {26{1'b1}})
                    state_d = TURN_LEFT;
            end
            TURN_LEFT: begin
                if (ctr_q == {25{1'b1}})
                    state_d = FORWARD;
            end
            default: state_d = FORWARD;
        endcase

        case (state_q)
            FORWARD: begin
                left = 8'h50;
                right = 8'h50;
            end
            BACKUP_RIGHT: begin
                left = 8'hB0;
                right = 8'hB0;
            end
            TURN_RIGHT: begin
                left = 8'hB0;
                right = 8'h50;
            end
            BACKUP_LEFT: begin
                left = 8'hB0;
                right = 8'hB0;
            end
            TURN_LEFT: begin
                left = 8'h50;
                right = 8'hB0;
            end
            default: begin
                left = 8'h80;
                right = 8'h80;
            end
        endcase
    end

    always @(posedge clk) begin
        if (rst) begin
            state_q <= FORWARD;
        end else begin
            state_q <= state_d;
        end
        ctr_q <= ctr_d;
        sl_q <= sl_d;
        sr_q <= sr_d;
    end

endmodule